'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var Inert = _interopDefault(require('inert'));
var H2O2 = _interopDefault(require('h2o2'));
var httpProxy = _interopDefault(require('http-proxy'));
var fs = _interopDefault(require('fs'));
var path = _interopDefault(require('path'));
var Logger = _interopDefault(require('@foundationjs/logger'));

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

const logger = new Logger('GatsbyController'); // Only the first instance will be functional in development

let nbInstance = 0;

const GatsbyController = function GatsbyController(options) {
  function initialize(_x) {
    return _initialize.apply(this, arguments);
  }

  function _initialize() {
    _initialize = _asyncToGenerator(function* (server) {
      // Gatsby needs two things to for a start: a static folder serving, and a proxy for development mode
      // In development, we can use options.forceStatic to run in production-like mode
      const production = process.env.NODE_ENV === 'production' || options.forceStatic; // Then we create the route handler gor Gatsby

      logger.info(`Initializing Gatsby Controller on path ${options.path} (Environment: ${process.env.NODE_ENV || 'development'}, Mode: ${production ? 'static' : 'webpack'})`);
      const handler = {};

      if (!production) {
        // In development, we establish the proxy
        handler.proxy = {
          uri: `http://${options.hmr.endpoint}:${options.hmr.port}/{gatsbyPath}`,
          passThrough: true
        };

        if (!nbInstance) {
          // We need to proxy commons.js if we want Gatsby to be functional
          server.route({
            method: 'GET',
            path: '/commons.js',
            handler: {
              proxy: {
                host: options.hmr.endpoint,
                port: options.hmr.port,
                passThrough: true
              }
            }
          }); // We also need to proxy socket.io, since Gatsby development uses the standard socket.io path
          // This means that any other socket.io used for Dazzled need to be on a different path

          server.route({
            method: ['POST', 'GET'],
            path: '/socket.io/{path*}',
            handler: {
              proxy: {
                host: options.hmr.endpoint,
                port: options.hmr.port,
                passThrough: true
              }
            }
          }); // Finally, we proxy the upgrade request is they start by /socket.io

          const wsProxy = httpProxy.createProxyServer({
            target: `http://${options.hmr.endpoint}:${options.hmr.port}`
          });
          wsProxy.on('error', error => logger.error(error));
          server.listener.on('upgrade', (req, socket, head) => {
            if (req.url.startsWith('/socket.io')) wsProxy.ws(req, socket, head);
          });
        } else {
          logger.warn('Multiple instances of Gatsby Controller have been found. ' + 'Only the first one will work in development out-of-the-box!');
          logger.warn(`To use this Gatsby instance in development, edit ${options.dir}/index.html` + ` and append '${options.path}' in front of '/commons.js'`);
        }

        logger.info(`Gatsby development mode listening on ${options.hmr.endpoint}:${options.hmr.port}`);
        logger.info(`To run Gatsby, use the following command:
  GATSBY_WEBPACK_PUBLICPATH=http://${options.hmr.endpoint}:${options.hmr.port}/` + ` gatsby develop -p ${options.hmr.port} --prefix-paths`);
        logger.info(`Don't forget to add this line to your gatsby-config.js file:
  pathPrefix: '${options.path}'`);
      } else {
        // In production, we create the static serving
        handler.directory = {
          path: options.dir
        }; // We also create redirect for dynamic routes, if any

        if (options.dynamicRoutes) {
          const dynamicRoutes = options.dynamicRoutes;
          Object.keys(dynamicRoutes).forEach(route => {
            const routeHandler = function routeHandler(req, h) {
              const filepath = req.url.pathname.split(options.path)[1];

              if (fs.existsSync(path.resolve(options.dir, `./${filepath}.html`))) {
                return h.file(path.resolve(options.dir, `./${filepath}.html`));
              }

              if (fs.existsSync(path.resolve(options.dir, `./${filepath}`, './index.html'))) {
                return h.file(path.resolve(options.dir, `./${filepath}`, './index.html'));
              }

              return h.file(`${options.dir}/${dynamicRoutes[route]}/index.html`);
            };

            server.route({
              method: 'GET',
              path: options.path + route,
              handler: routeHandler
            });
            logger.info(`Added dynamic route for path ${route} -> ${dynamicRoutes[route]}`);
          });
        }
      }

      nbInstance += 1;
      const mainPath = options.path + (options.path.endsWith('/') ? '{gatsbyPath*}' : '/{gatsbyPath*}'); // Then we create the route

      server.route({
        method: '*',
        path: mainPath,
        handler
      });
      server.route({
        method: ['POST', 'GET'],
        path: mainPath,
        handler
      });
      logger.info(`Gatsby static site served on ${options.path} (${options.dir})`);
    });
    return _initialize.apply(this, arguments);
  }

  return {
    initialize,
    plugins: [Inert, H2O2]
  };
};

exports.default = GatsbyController;
